'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved
'
' @file   GPSdemo1-spi.bas
' @target PICBASIC PRO
'
' This demo program reads latitude and longitude values from a GPS receiver
' and displays them in the terminal window using the DEBUG command. The 4800
' baud serial output from the GPS receiver is connected to the SERIN pin of
' the uM-FPU V3.1 chip. The sentence count and sentence type is displayed
' for each record, and the longitude and latitude values are displayed for the
' GPRMC sentence. The sentence types will vary depending on the GPS receiver
' used. The following example shows the output from a Garmin receiver:
'
'   GPSdemo1
'   uM-FPU V3.1.0
'
'   1:GPGLL
'   2:GPBOD
'   3:GPVTG
'   4:PGRME
'   5:PGRMZ
'   6:PGRMM
'   7:HCHDG
'   8:GPRTE
'   9:GPRMC: Lat = 7622.4609, Long = 7622.4609
'   (etc.)
' ...
'
' See Application Note 36 - Reading GPS Data
'
' @author Cam Thompson, Micromega Corporation
' @version
'	September 18, 2007
'	- modified for PICBASIC PRO
'   March 15, 2007
'   - original version
'==============================================================================

'-------------------- PIC oscillator speed ------------------------------------
define      OSC 20                      ' specify the speed of the oscillator
OSC_SPEED   con 20                      ' define PICBASIC constant

'-------------------- debug definitions ---------------------------------------
define      DEBUG_REG   PORTC
define      DEBUG_BIT   6
define      DEBUG_BAUD  19200
define      DEBUG_MODE  05

'-------------------- uM-FPU pin definitions ----------------------------------
FpuClk          var     PORTC.3			' SPI SCLK (uM-FPU SCLK)
FpuIn           var     PORTC.4			' SPI MISO (uM-FPU SOUT)
FpuOut          var     PORTC.5			' SPI MOSI (uM-FPU SIN)

include     "umfpuV3-spi.bas"           ' include uM-FPU V3.1 support routines

'-------------------- uM-FPU register definitions -----------------------------
Latitude        con     8				' latitude
Longitude       con     9				' longitude

'-------------------- variables -----------------------------------------------
cnt             var     Word            ' counter

'-------------------- definitions ---------------------------------------------
GPS_OK          con     $80             ' GPS sentence OK status

'==============================================================================
'-------------------- initialization ------------------------------------------
'==============================================================================

Reset:
  DEBUG 13, 10, 13, 10, "GPSdemo1", 13, 10

  GOSUB Fpu_Reset						' reset the FPU hardware
  IF fpu_status <> SYNC_CHAR THEN		' check for synchronization
    DEBUG "uM-FPU not detected"
    END
  ELSE
    GOSUB Print_Version					' display the uM-FPU version number
    DEBUG 13, 10
  ENDIF

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:

  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [F_SEROUT, 0, 5]	' set 4800 baud
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [F_SERIN, 4]		' enable NMEA

  FOR cnt = 1 TO 200
    DEBUG 13, 10, DEC cnt, ":"
    SHIFTOUT FpuOut, FpuClk, MSBFIRST, [F_SERIN, 6]
    GOSUB Fpu_ReadStatus

    IF fpu_status = GPS_OK THEN
      GOSUB Print_FpuStringSel

      SHIFTOUT FpuOut, FpuClk, MSBFIRST, [STRCMP, "GPRMC", 0]
      GOSUB Fpu_ReadStatus

      IF fpu_status_Zero THEN
        SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, Latitude, STRFIELD, 4, STRTOF, FSET0]
        SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, Longitude, STRFIELD, 6, STRTOF, FSET0]
        SHIFTOUT FpuOut, FpuClk, MSBFIRST, [STRSET, 0]
        SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, Latitude]
        DEBUG 13, 10, "*** Lat = "
        GOSUB Print_Float
        SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, Longitude]
        DEBUG ", Long = "
        GOSUB Print_Float
      ENDIF

    ELSE
      DEBUG 13, 10, 13, 10, "*** data error ***", 13, 10
    ENDIF
  NEXT

Done:
  DEBUG 13, 10, "Done.", 13, 10
  END

'-------------------- print string selection ----------------------------------

Print_FpuStringSel:
  GOSUB Fpu_Wait						' wait until uM-FPU is ready
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [READSEL]
  PAUSEUS ReadDelay
  GOTO Print_FpuStr2
  
