'==============================================================================
' Copyright  Micromega Corporation 2004-2007. All rights reserved.
'
' @file		sample-spi.bsp
' @target	PICBASIC PRO
'
' Demonstration routine for uM-FPU V3.1 using an SPI interface. The demo
' program takes an integer value representing the diameter of a circle in
' centimeters, converts the value to inches, then calculates the circumference
' and area in inches and square inches.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	September 18, 2007
'	- modified for uM-FPU V3.1
'	V3.0 - October 23, 2006
'	- modified for uM-FPU V3
'   V2.0 - January 17, 2005
'   - modified for uM-FPU V2 and PICBASIC PRO compiler
'   V1.0 - April 29, 2004
'   - original version
'==============================================================================

'-------------------- PIC oscillator speed ------------------------------------
' The oscillator speed must be defined to ensure proper timing constants
' for the uM-FPU routines.

define      OSC 20                      ' specify the speed of the oscillator
OSC_SPEED   con 20                      ' define PicBasic constant

'-------------------- debug definitions ---------------------------------------
' The serial pin and baud rate for the DEBUG command are specified with the
' following defines. These should be changed to suit your application.

define      DEBUG_REG   PORTC
define      DEBUG_BIT   6
define      DEBUG_BAUD  19200
define      DEBUG_MODE  0

'-------------------- uM-FPU pin definitions ----------------------------------
FpuClk          var     PORTC.3			' SPI SCLK (uM-FPU SCLK)
FpuIn           var     PORTC.4			' SPI MISO (uM-FPU SOUT)
FpuOut          var     PORTC.5			' SPI MOSI (uM-FPU SIN)

include     "umfpuV3-spi.bas"           ' include the uM-FPU V3 support routines

'-------------------- uM-FPU register definitions -----------------------------

DiameterIn      con     1               ' diameter in inches
Circumference   con     2               ' circumference
Area            con     3               ' area
Pi              con     4               ' constant pi

'-------------------- variables -----------------------------------------------

diameterCm      var     byte            ' diameter in centimeters

'------------------------------------------------------------------------------
' The following example takes an integer value representing the diameter
' of a circle in centimeters, converts the value to inches and calculates
' the circumference and area in inches and square inches.
'------------------------------------------------------------------------------

'=============================================================================
'-------------------- initialization -----------------------------------------
'=============================================================================

Reset:
  DEBUG 13, 10, 13, 10, "Conversion Example"
  DEBUG 13, 10, "------------------", 13, 10

  '------------------------------------
  ' Reset the uM-FPU and print version.
  '------------------------------------

  GOSUB Fpu_Reset                       ' reset the FPU hardware
  IF fpu_status <> SYNC_CHAR THEN
    DEBUG "uM-FPU not detected."
    END
  ELSE
    GOSUB Print_Version                 ' display the uM-FPU version number
    DEBUG 13, 10
  ENDIF

  '-----------------------------------------------------
  ' Load constant for later use.
  '-----------------------------------------------------
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, Pi, LOADPI, FSET0]

'=============================================================================
'-------------------- main routine -------------------------------------------
'=============================================================================

Main:
  '----------------------------------------------------------------------------
  ' Get diameter in centimeters.  The value would typically come from a sensor
  ' reading, but in this example an assumed value of 25 is used.
  '----------------------------------------------------------------------------
  diameterCm = 25
  DEBUG 13, 10, "Diameter (cm):       ", DEC diameterCm

  '------------------------------
  ' diameterIn = diameterCm / 2.54
  '------------------------------
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, DiameterIn, FSETI, diameterCm, FCNV, 5]
  DEBUG 13, 10, "Diameter (in.):      "
  GOSUB Print_Float

  '------------------------------
  ' circumference = diameter * pi
  '------------------------------
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, Circumference, FSET, DiameterIn, FMUL, Pi]
  DEBUG 13, 10, "Circumference (in.): "
  GOSUB Print_Float

  '----------------------------
  'area = (diameter / 2)^2 * pi
  '----------------------------
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, Area, FSET, DiameterIn, FDIVI, 2, FMUL, Area, FMUL, Pi]
  DEBUG 13, 10, "Area (sq.in.):       "
  GOSUB Print_Float

  DEBUG 13, 10, "Done.", 13, 10         'end of program
  END
